#    Massive Enumeration Toolset
#    Copyright (C) 2005  Petko Petkov (GNUCITIZEN) ppetkov@gnucitizen.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import re

from getopt import gnu_getopt as getopt
from getopt import GetoptError


class CmdError(GetoptError): 
	pass

class CmdExpect:
	__options = {}

	def remove(self, option):
		del self.__options[option]

	def __handle(self, option, processor = None, parameters = None):
		ill_chars = [':']

		for c in ill_chars:
			if option.find(c) > 0:
				raise CmdError, 'illegel character %s' % c

		self.__options[option] = (processor, parameters)

	def flag(self, option):
		self.__handle(option)

	def __p_integer(self, option, value, parameters):
		min = parameters[0]
		max = parameters[1]

		try:
			number = int(value)
		except ValueError:
			raise CmdError, 'option %s is not a valid integer' % option

		if (min != None and number < min) or (max != None and number > max):
			raise CmdError, 'option %s is not in range (%s:%s)' % (option, min, max)

		return number

	def integer(self, option, min = None, max = None):
		self.__handle(option, self.__p_integer, (min, max))

	def __p_url(self, option, value, parameters):
		p = re.compile('\w+://(?:[^:].)+(?::\d+/?)?.*')

		if not p.match(value):
			raise CmdError, 'option %s is not a valid url' % option

		return value
		
	def url(self, option):
		self.__handle(option, self.__p_url)

	def __p_file(self, option, value, parameters):
		mod = parameters[0]

		try:
			fd = file(value, mod)
		except IOError, e:
			raise CmdError, 'cannot open file "%s" in mod "%s"' % (value, mod)

		return fd

	def file(self, option, mod):
		self.__handle(option, self.__p_file, (mod))

	def __p_range(self, option, value, parameters):
		l_part = None
		r_part = None

		min = parameters[0]
		max = parameters[1]

		parts = value.split(':')

		if len(parts) == 2:
			l_part = parts[0]
			r_part = parts[1]

		elif len(parts) == 1:
			l_part = parts[0]
			r_part = parts[0] 

		else:
			raise CmdError, 'option %s is not a valid range' % option

		if not l_part:
			l_part = min

		if not r_part:
			r_part = max

		try:
			if l_part: l_part = int(l_part)
			if r_part: r_part = int(r_part)

		except ValueError:
			raise CmdError, 'option %s is not a valid range' % option

		if l_part != None and r_part != None and l_part > r_part:
			raise CmdError, 'option %s is not a valid range' % option

		if (min != None and l_part < min) or (max != None and r_part > max):
			raise CmdError, 'option %s is not in range (%s:%s)' % (option, min, max)

		return l_part, r_part

	def range(self, option, min = None, max = None):
		self.__handle(option, self.__p_range, (min, max))

	def __p_list(self, option, value, parameters):
		parts = [x.strip() for x in value.split(',')]

		if len(parts) < 1:
			raise CmdError, 'option %s is not a valid list' % option

		return parts

	def list(self, option):
		self.__handle(option, self.__p_list)

	def __p_input(self, option, value, parameters):
		return value

	def input(self, option):
		self.__handle(option, self.__p_input)

	def __p_option(self, option, value, parameters):
		if value in parameters:
			return value
		else:
			raise CmdError, 'valid values for option %s are (%s)' % (option, '|'.join(parameters))

	def option(self, option, options):
		self.__handle(option, self.__p_option, options)

	def getOptionStrings(self):
		short_opts = ''
		long_opts  = [] 

		for opt in self.__options.keys():
			proc = self.__options[opt][0]

			if proc == None:
				if len(opt) > 1:
					long_opts.append(opt)
				else:
					short_opts += opt
			else:
				if len(opt) > 1:
					long_opts.append(opt + '=')
				else:
					short_opts += opt + ':'

		return short_opts, long_opts

	def process(self, option, value):
		if not self.__options.has_key(option):
			return None	
		else:
			proc = self.__options[option][0]
			parameters = self.__options[option][1]

			if not proc:
				return option, value
			else:
				return option, proc(option, value, parameters)

class CmdOptions:
	__argv = [] 

	expect = CmdExpect()

	def __init__(self, argv = None):
		if argv:
			self.__argv = argv

	def process(self, argv = None):
		if not argv:
			argv = self.__argv

		short_opts, long_opts = self.expect.getOptionStrings()

		try:
			opts, args = getopt(argv, short_opts, long_opts) 
		except GetoptError, e:
			raise CmdError, str(e)

		cmdopts = []

		for opt, val in opts:
			if opt[:2] == '--':
				opt = opt[2:]
			else:
				opt = opt[1:]

			result = self.expect.process(opt, val)

			if result:
				cmdopts.append(result)

		return cmdopts, args
